/*
 * iGeom by LInE
 * Free software to student private data
 *
 * http://www.matematica.br/igeom
 * http://www.usp.br/line
 *
 * Prepare string to build GEO file in iGeomJS
 * 
 * ./app/core/models/application/actions/action-manager.js
 * @version 2023/07/14: Fixed reading GEO file with midpoint
 */

import { xor } from "lodash";
import { browser } from "../../../application/browser";

export class ActionManager {

  // @calledby app/core/drawers/layer.js: constructor(id, konvaLayer): this.actionManager = new ActionManager();
  constructor () {
    console.log("action-manager.js!constructor()"); //D console.trace();
    this.actions = [];
    }

  push (action) {
    // "TypeError: cyclic object value" : console.log("action-manager.js!push(action): action=" + JSON.stringify(action));
    var aux = "action(id,type,definition,label,genericObject)=" + (action && action!=undefined ? 
      "(" + action.id + "," + action.type + ",'" + action.definition + "','" + action.label + "'," + action.genericObject + ")" : "<empty>");
    console.log("action-manager.js!push(action): " + aux);
    this.actions.push(action); // "action" is JSON like: {"id":2,"type":0,"definition":... ,"elementClass":0}
    }

  remove (genericObject) {
    const action = this.actions.find(action => action.genericObject == genericObject);
    const index = this.actions.indexOf(action);
    console.log("action-manager.js!remove(genericObject): action=" + action + ", index=" + index);
    this.actions.splice(index, 1);
    }

  pop () {
    throw "Not implemented";
    }

  clear () {
    console.log("action-manager.js!clear()");
    this.actions = [];
    }

  load (actions) {
    console.log("action-manager.js!load(actions)");
    this.actions = actions;
    }

  // Generate GEO file
  save () {
    var strLine = "";
    // app/core/application/browser.js!construct(): build information about the browser to be used in GEO file bellow 'browser.osName'
    const LINES = [
      `# igeom: http://www.matematica.br!
[ .: iGeom : Geometria Interativa na Internet :. ]!
[ versao: 1.0.0 ]!
[ PC = ${browser.appName  }; DD = 0 ]!
[[${new Date()  }; ${browser.osName  }]]!
[0:1.1, 1:3,  - iGeom versao 1.0.0]!\n`
    ];
    console.log("action-manager.js!save(): initial");
    this.actions.forEach(action => {
      try {
        action.rehydrate(); // in app/core/models/application/actions/action.js
        strLine = action.toString();
        LINES.push(strLine);
      } catch (error) { console.log("action-manager.js!save(): error " + error + "\n* " + strLine); }
      });
    let file = new Blob([...LINES], {
      type: "text/plain;charset=utf-8"
      });
    console.log("action-manager.js!save(): final");
    return file;
    }

  } // export class ActionManager